% x = dgpnlp(K,P,x0)
%
% Applies local NLP descent to x0 using the unconstrained quartic formulation
% K is the dimension of the space, and P is an nxn partial matrix_type
% both x and x0 are K x n matrices
% If x0 is not given, it is initialized to random

function [x,ret] = dgpnlp(K,P,x0)
  [n,n] = size(P);
  x = [];
  ret.x = x;
  
  if n < 2
    fprintf('dgpnlp: ERROR: n < 2\n');
    return
  end
  
  if K < 1
    fprintf('dgpnlp: ERROR: K < 1\n');
    return
  end
  
  if (nargin < 3)
    x0 = rand(K,n);
  end
  
  objfun = @ (a) dgpobj(K,P,a);
  vecx0 = rlz2vec(x0,K,n);

  %% EITHER: using optim package
  %[vecx, f, cvg, out] = nonlin_min(objfun, vecx0);
  %ret.info = cvg;
  %ret.iter = out;
  %ret.nf = -1;
  %ret.lambda = [];
  %% END EITHER
  
  %% OR: using sqp()
  mitn = 5;
  tol = 1e-2;
  M = sum(sum(P));
  LB = []; %-M*ones(n*K,1);
  UB = []; % M*ones(n*K,1);
  [vecx,f,info,iter,nf,lambda] = sqp(vecx0,objfun,[],[],LB,UB,mitn,tol);
  % solution back into matrix (realization) form
  ret.info = info;
  ret.iter = iter;
  ret.nf = nf;
  ret.lambda = lambda;
  %% END OR
  
  x = vec2rlz(vecx,K,n);
  ret.x = x;
  ret.f = f;
end

function f = dgpobj(K,P,vecx0)
  [n,n] = size(P);
  f = 0;
  for i = 1:n-1
    for j = i+1:n
      if P(i,j) ~= -1
        t = 0;
        for k = 1:K
          hi = (i-1)*K + k;
          hj = (j-1)*K + k;
          t = t + (vecx0(hi) - vecx0(hj))^2;       
        end
        t = (t - P(i,j)^2)^2;
        f = f + t;
      end
    end
  end
end
 
